searchData={"items":[{"type":"module","title":"Ymlr","doc":"Encodes data into YAML documents using the `Ymlr.Encoder` protocol.\nEvery document starts with a separator (\"---\") and can be enhanced with comments.","ref":"Ymlr.html"},{"type":"function","title":"Ymlr.document/2","doc":"Encodes a given data as YAML document with a separator (\"---\") at the beginning.\n\nOptinally you can pass a tuple with comment(s) and data as first argument.","ref":"Ymlr.html#document/2"},{"type":"function","title":"Options - Ymlr.document/2","doc":"* `atoms` - when set to `true`, encodes atom map keys with a leading colon.\n* `sort_maps` - when set to `true`, sorts map by key-values.","ref":"Ymlr.html#document/2-options"},{"type":"function","title":"Examples - Ymlr.document/2","doc":"iex> Ymlr.document(%{a: 1})\n    {:ok, \"---\\na: 1\\n\"}\n\n    iex> Ymlr.document(%{a: 1}, atoms: true)\n    {:ok, \"---\\n:a: 1\\n\"}\n\n    iex> Ymlr.document({\"comment\", %{a: 1}})\n    {:ok, \"---\\n# comment\\na: 1\\n\"}\n\n    iex> Ymlr.document({[\"comment 1\", \"comment 2\"], %{a: 1}})\n    {:ok, \"---\\n# comment 1\\n# comment 2\\na: 1\\n\"}\n\n    iex> Ymlr.document(Map.new(1..33, &{&1, &1}), sort_maps: true)\n    {:ok, \"---\\n1: 1\\n2: 2\\n3: 3\\n4: 4\\n5: 5\\n6: 6\\n7: 7\\n8: 8\\n9: 9\\n10: 10\\n11: 11\\n12: 12\\n13: 13\\n14: 14\\n15: 15\\n16: 16\\n17: 17\\n18: 18\\n19: 19\\n20: 20\\n21: 21\\n22: 22\\n23: 23\\n24: 24\\n25: 25\\n26: 26\\n27: 27\\n28: 28\\n29: 29\\n30: 30\\n31: 31\\n32: 32\\n33: 33\\n\"}","ref":"Ymlr.html#document/2-examples"},{"type":"function","title":"Ymlr.document!/2","doc":"Encodes a given data as YAML document with a separator (\"---\") at the beginning. Raises if it cannot be encoded.\n\nOptinally you can pass a tuple with comment(s) and data as first argument.","ref":"Ymlr.html#document!/2"},{"type":"function","title":"Options - Ymlr.document!/2","doc":"* `atoms` - when set to `true`, encodes atom map keys with a leading colon.\n* `sort_maps` - when set to `true`, sorts map by key-values.","ref":"Ymlr.html#document!/2-options"},{"type":"function","title":"Examples - Ymlr.document!/2","doc":"iex> Ymlr.document!(%{a: 1})\n    \"---\\na: 1\\n\"\n\n    iex> Ymlr.document!(%{a: 1}, atoms: true)\n    \"---\\n:a: 1\\n\"\n\n    iex> Ymlr.document!({\"comment\", %{a: 1}})\n    \"---\\n# comment\\na: 1\\n\"\n\n    iex> Ymlr.document!({[\"comment 1\", \"comment 2\"], %{a: 1}})\n    \"---\\n# comment 1\\n# comment 2\\na: 1\\n\"\n\n    iex> Ymlr.document!(Map.new(1..33, &{&1, &1}), sort_maps: true)\n    \"---\\n1: 1\\n2: 2\\n3: 3\\n4: 4\\n5: 5\\n6: 6\\n7: 7\\n8: 8\\n9: 9\\n10: 10\\n11: 11\\n12: 12\\n13: 13\\n14: 14\\n15: 15\\n16: 16\\n17: 17\\n18: 18\\n19: 19\\n20: 20\\n21: 21\\n22: 22\\n23: 23\\n24: 24\\n25: 25\\n26: 26\\n27: 27\\n28: 28\\n29: 29\\n30: 30\\n31: 31\\n32: 32\\n33: 33\\n\"","ref":"Ymlr.html#document!/2-examples"},{"type":"function","title":"Ymlr.documents/2","doc":"Encodes a given list of data as \"---\" separated YAML documents.","ref":"Ymlr.html#documents/2"},{"type":"function","title":"Options - Ymlr.documents/2","doc":"* `atoms` - when set to `true`, encodes atom map keys with a leading colon.\n* `sort_maps` - when set to `true`, sorts map by key-values.","ref":"Ymlr.html#documents/2-options"},{"type":"function","title":"Examples - Ymlr.documents/2","doc":"iex> Ymlr.documents([%{a: 1}])\n    {:ok, \"---\\na: 1\\n\"}\n\n    iex> Ymlr.documents([%{a: 1}], atoms: true)\n    {:ok, \"---\\n:a: 1\\n\"}\n\n    iex> Ymlr.documents([%{a: 1}, %{b: 2}])\n    {:ok, \"---\\na: 1\\n\\n---\\nb: 2\\n\"}\n\n    iex> Ymlr.documents(%{a: \"a\"})\n    {:error, \"The given argument is not a list of documents. Use document/1, document/2, document!/1 or document!/2 for a single document.\"}","ref":"Ymlr.html#documents/2-examples"},{"type":"function","title":"Ymlr.documents!/2","doc":"Encodes a given list of data as \"---\" separated YAML documents. Raises if it cannot be encoded.","ref":"Ymlr.html#documents!/2"},{"type":"function","title":"Options - Ymlr.documents!/2","doc":"* `atoms` - when set to `true`, encodes atom map keys with a leading colon.\n* `sort_maps` - when set to `true`, sorts map by key-values.","ref":"Ymlr.html#documents!/2-options"},{"type":"function","title":"Examples - Ymlr.documents!/2","doc":"iex> Ymlr.documents!([%{a: 1}])\n    \"---\\na: 1\\n\"\n\n    iex> Ymlr.documents!([%{a: 1}], atoms: true)\n    \"---\\n:a: 1\\n\"\n\n    iex> Ymlr.documents!([%{a: 1}, %{b: 2}])\n    \"---\\na: 1\\n\\n---\\nb: 2\\n\"\n\n    iex> Ymlr.documents!(%{a: \"a\"})\n    ** (ArgumentError) The given argument is not a list of documents. Use document/1, document/2, document!/1 or document!/2 for a single document.","ref":"Ymlr.html#documents!/2-examples"},{"type":"type","title":"Ymlr.document/0","doc":"","ref":"Ymlr.html#t:document/0"},{"type":"module","title":"Ymlr.Encode","doc":"This module implements the logic of encoding scalars.\n\n## Strings and Characters\n\n#","ref":"Ymlr.Encode.html"},{"type":"module","title":"Printable Characters - Ymlr.Encode","doc":"The YAML spec defines a set of printable characters `c-printable` (see\nhttps://yaml.org/spec/1.2.2/#character-set). All these characters can\ntheoretically be left alone when encoding a string.\n\n#","ref":"Ymlr.Encode.html#module-printable-characters"},{"type":"module","title":"Escape Characters - Ymlr.Encode","doc":"The YAML spec also defines a set of escape charactesr `c-ns-esc-char` (see\nhttps://yaml.org/spec/1.2.2/#57-escaped-characters). Some of these chars are\nalso in the printable range `c-printable`. Being in `c-printable` means they\ncould be left alone. I.e. there would be no need to encode them as escape\nchars. However, we think in certain cases, escape characters are more\nreader friendly than the actual characters. An example is the \"next line\"\ncharacter (`U+0085` or `\\N`). It is part of `c-printable`. However, on the\nscreen this character cannot be distinguished from a simple \"line feed\"\ncharacter (`U+000A` or `\\n`). Therefore all characters in `c-ns-esc-char` with\nthe exception of `\\n` and `\\t` are always encoded using their escape character.\n\n#","ref":"Ymlr.Encode.html#module-escape-characters"},{"type":"module","title":"Other 8-bit Unicode Characters - Ymlr.Encode","doc":"Any 8-bit unicode character that neither a printable nor an escape character\nhas to be encoded using one of the three unicode escape characters \\x, \\u or\n\\U (i.e. \\xXX, \\u00XX or \\U000000XX).\n\n#","ref":"Ymlr.Encode.html#module-other-8-bit-unicode-characters"},{"type":"module","title":"Double Quotes for Escape Characters - Ymlr.Encode","doc":"Printable Characters can be encoded unquoted, single-quoted or double-quoted.\nEscape characters require double quotes.\n\n#","ref":"Ymlr.Encode.html#module-double-quotes-for-escape-characters"},{"type":"module","title":"Chars with Special Treatments - Ymlr.Encode","doc":"##","ref":"Ymlr.Encode.html#module-chars-with-special-treatments"},{"type":"module","title":"Chars `\\n` and `\\t` - Ymlr.Encode","doc":"These two characters are never converted to their escape characters.\nOne exception: If the given string is literally just a newline, we\nencode it as \"\\n\" (double quotes required for escape chars) rather than a\nsingle newline.\n\n##","ref":"Ymlr.Encode.html#module-chars-n-and-t"},{"type":"module","title":"Chars `\"` and `\\` - Ymlr.Encode","doc":"These two characters have escape characters (`\\\"` and `\\\\`) but they are also\npart of the of the printable character range `c-printable` and they have a\nwell-defined presentation on the screen. Ocurrance of these characters don't\nenforce double-quotes but if they occur within a string that for other reasons\nrequires double-quotes, they need to be escaped.\n\n#","ref":"Ymlr.Encode.html#module-chars-and"},{"type":"module","title":"Implemented Decision Logic - Ymlr.Encode","doc":"First matching rule is applied:\n\n1. Char is `\\t` or `\\n` => leave alone\n1. Char is `\"` or `\\` => if within double quotes, escape. Otherwise leave alone.\n1. Char has an escape character (i.e. is part of `c-ns-esc-char`) => force double quotes and encode as escape character\n1. Char is a printable character => leave alone\n1. Char is a non-printable character => force double quotes and encode as \\xXX (only 8-bit supported for now)","ref":"Ymlr.Encode.html#module-implemented-decision-logic"},{"type":"function","title":"Ymlr.Encode.atom/1","doc":"","ref":"Ymlr.Encode.html#atom/1"},{"type":"macro","title":"Ymlr.Encode.force_double_quote/1","doc":"","ref":"Ymlr.Encode.html#force_double_quote/1"},{"type":"macro","title":"Ymlr.Encode.is_printable/1","doc":"","ref":"Ymlr.Encode.html#is_printable/1"},{"type":"function","title":"Ymlr.Encode.list/3","doc":"","ref":"Ymlr.Encode.html#list/3"},{"type":"function","title":"Ymlr.Encode.map/3","doc":"","ref":"Ymlr.Encode.html#map/3"},{"type":"function","title":"Ymlr.Encode.number/1","doc":"","ref":"Ymlr.Encode.html#number/1"},{"type":"macro","title":"Ymlr.Encode.requires_unicode_escape/1","doc":"","ref":"Ymlr.Encode.html#requires_unicode_escape/1"},{"type":"function","title":"Ymlr.Encode.string/2","doc":"","ref":"Ymlr.Encode.html#string/2"},{"type":"function","title":"Ymlr.Encode.to_s/2","doc":"Encodes the given data as YAML string.","ref":"Ymlr.Encode.html#to_s/2"},{"type":"function","title":"Examples - Ymlr.Encode.to_s/2","doc":"iex> Ymlr.Encode.to_s(%{a: 1, b: 2})\n    {:ok, \"a: 1\\nb: 2\"}","ref":"Ymlr.Encode.html#to_s/2-examples"},{"type":"function","title":"Ymlr.Encode.to_s!/2","doc":"Encodes the given data as YAML string. Raises if it cannot be encoded.","ref":"Ymlr.Encode.html#to_s!/2"},{"type":"function","title":"Examples - Ymlr.Encode.to_s!/2","doc":"iex> Ymlr.Encode.to_s!(%{})\n    \"{}\"\n\n    iex> Ymlr.Encode.to_s!(%{a: 1, b: 2})\n    \"a: 1\\nb: 2\"\n\n    iex> Ymlr.Encode.to_s!(%{\"a\" => \"a\", \"b\" => :b, \"c\" => \"true\", \"d\" => \"100\"})\n    \"a: a\\nb: b\\nc: 'true'\\nd: '100'\"","ref":"Ymlr.Encode.html#to_s!/2-examples"},{"type":"protocol","title":"Ymlr.Encoder","doc":"Protocol controlling how a value is encoded to YAML.","ref":"Ymlr.Encoder.html"},{"type":"protocol","title":"Deriving - Ymlr.Encoder","doc":"The protocol allows leveraging the Elixir's `@derive` feature to simplify\nprotocol implementation in trivial cases. Accepted options are:\n\n  * `:only` - encodes only values of specified keys.\n  * `:except` - If set to a list of fields, all struct fields except specified\n    keys are encoded. If set to `:defaults`, all fields are encoded, except\n    the ones equaling their default value as defined in the struct.\n\nBy default all keys except the `:__struct__` key are encoded.","ref":"Ymlr.Encoder.html#module-deriving"},{"type":"protocol","title":"Example - Ymlr.Encoder","doc":"Let's assume a presence of the following struct:\n\n```\ndefmodule Test do\n  defstruct [:foo, :bar, :baz]\nend\n```\n\nIf we were to call `@derive Ymlr.Encoder` just before `defstruct`, an\nimplementation similar to the following implementation would be generated:\n\n```\ndefimpl Ymlr.Encoder, for: Test do\n  def encode(data, idnent_level) do\n    data\n    |> Map.take(unquote([:foo, :bar, :baz]))\n    |> Ymlr.Encode.map(idnent_level)\n  end\nend\n```\n\n#","ref":"Ymlr.Encoder.html#module-example"},{"type":"protocol","title":"Limit fields using the `only` Option - Ymlr.Encoder","doc":"We can limit the fields being encoded using the `:only` option:\n\n```\ndefmodule Test do\n  @derive {Ymlr.Encoder, only: [:foo]}\n  defstruct [:foo, :bar, :baz]\nend\n```\n\nThis would generate an implementation similar to the following:\n\n```\ndefimpl Ymlr.Encoder, for: Test do\n  def encode(data, idnent_level) do\n    data\n    |> Map.take(unquote([:foo]))\n    |> Ymlr.Encode.map(idnent_level)\n  end\nend\n```\n\n#","ref":"Ymlr.Encoder.html#module-limit-fields-using-the-only-option"},{"type":"protocol","title":"Exclude fields using the `except` Option - Ymlr.Encoder","doc":"We can exclude the fields being encoded using the `:except` option:\n\n```\ndefmodule Test do\n  @derive {Ymlr.Encoder, except: [:foo]}\n  defstruct [:foo, :bar, :baz]\nend\n```\n\nThis would generate an implementation similar to the following:\n\n```\ndefimpl Ymlr.Encoder, for: Test do\n  def encode(data, idnent_level) do\n    data\n    |> Map.take(unquote([:bar, :baz]))\n    |> Ymlr.Encode.map(idnent_level)\n  end\nend\n```\n\nWe can exclude the fields being left at their defaults by passing `except:\n:defaults`:\n\n```\ndefmodule TestExceptDefaults do\n  @derive {Ymlr.Encoder, except: :defaults}\n  defstruct [:foo, bar: 1, baz: :ok]\nend\n```\n\nThis would generate an implementation similar to the following:\n\n```\niex> Ymlr.document!(%TestExceptDefaults{foo: 1, bar: 1, baz: :error})\n\"baz: error\\nfoo: 1\"\n```","ref":"Ymlr.Encoder.html#module-exclude-fields-using-the-except-option"},{"type":"function","title":"Ymlr.Encoder.encode/3","doc":"Encodes the given data to YAML.","ref":"Ymlr.Encoder.html#encode/3"},{"type":"type","title":"Ymlr.Encoder.opts/0","doc":"","ref":"Ymlr.Encoder.html#t:opts/0"},{"type":"type","title":"Ymlr.Encoder.t/0","doc":"All the types that implement this protocol.","ref":"Ymlr.Encoder.html#t:t/0"},{"type":"extras","title":"ymlr - A YAML Encoder for Elixir","doc":"# ymlr - A YAML Encoder for Elixir\n\nymlr - A YAML encoder for Elixir.\n\n[![Module Version](https://img.shields.io/hexpm/v/ymlr.svg)](https://hex.pm/packages/ymlr)\n[![Coverage Status](https://coveralls.io/repos/github/ufirstgroup/ymlr/badge.svg?branch=main)](https://coveralls.io/github/ufirstgroup/ymlr?branch=main)\n[![Last Updated](https://img.shields.io/github/last-commit/ufirstgroup/ymlr.svg)](https://github.com/ufirstgroup/ymlr/commits/main)\n\n[![Build Status Code Qualits](https://github.com/ufirstgroup/ymlr/actions/workflows/code_quality.yaml/badge.svg)](https://github.com/ufirstgroup/ymlr/actions/workflows/code_quality.yaml)\n[![Build Status Elixir](https://github.com/ufirstgroup/ymlr/actions/workflows/elixir_matrix.yaml/badge.svg)](https://github.com/ufirstgroup/ymlr/actions/workflows/elixir_matrix.yaml)\n\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ymlr/)\n[![Total Download](https://img.shields.io/hexpm/dt/ymlr.svg)](https://hex.pm/packages/ymlr)\n[![License](https://img.shields.io/hexpm/l/ymlr.svg)](https://github.com/ufirstgroup/ymlr/blob/main/LICENSE)","ref":"readme.html"},{"type":"extras","title":"Installation - ymlr - A YAML Encoder for Elixir","doc":"The package can be installed by adding `ymlr` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ymlr, \"~> 5.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Examples - ymlr - A YAML Encoder for Elixir","doc":"See The usage livebook `usage.livemd` for more detailed examples.\n\n#","ref":"readme.html#examples"},{"type":"extras","title":"Encode a single document - optionally with comments: - ymlr - A YAML Encoder for Elixir","doc":"```elixir\n  iex> Ymlr.document!(%{a: 1})\n  \"\"\"\n  ---\n  a: 1\n  \"\"\"\n\n  iex> Ymlr.document!({\"comment\", %{a: 1}})\n  \"\"\"\n  ---\n  # comment\n  a: 1\n  \"\"\"\n\n  iex> Ymlr.document!({[\"comment 1\", \"comment 2\"], %{\"a\" => \"a\", \"b\" => :b, \"c\" => \"true\", \"d\" => \"100\"}})\n  \"\"\"\n  ---\n  # comment 1\n  # comment 2\n  a: a\n  b: b\n  c: 'true'\n  d: '100'\n  \"\"\"\n```\n\n#","ref":"readme.html#encode-a-single-document-optionally-with-comments"},{"type":"extras","title":"Encode a multiple documents - ymlr - A YAML Encoder for Elixir","doc":"```elixir\niex> Ymlr.documents!([%{a: 1}, %{b: 2}])\n\"\"\"\n---\na: 1\n\n---\nb: 2\n\"\"\"\n```","ref":"readme.html#encode-a-multiple-documents"},{"type":"extras","title":"Support for atoms - ymlr - A YAML Encoder for Elixir","doc":"By default, atoms as map keys are encoded as strings (without the leading\ncolon). If you want atoms to be encoded with a leading colon in order to be able\nto parse it later using [`YamlElixir`'s `atoms`\noption](https://hexdocs.pm/yaml_elixir/readme.html#support-for-atoms), you can\npass `atoms: true` as second argument to any of the `Ymlr` module's functions:\n\n```elixir\niex> Ymlr.document!(%{a: 1}, atoms: true)\n\"\"\"\n---\n:a: 1\n\"\"\"\n```\n\n#","ref":"readme.html#support-for-atoms"},{"type":"extras","title":"Encode maps with keys sorted - ymlr - A YAML Encoder for Elixir","doc":"Maps in elixir, implemented by erlang `:maps`, internally are `flatmap`s or `hashmap`s by size.\nLarge maps will be encoded in strange order.\n\n```elixir\niex> 1..33 |> Map.new(&{&1, &1})|> Ymlr.document!() |> IO.puts\n---\n4: 4\n25: 25\n8: 8\n...\n\n```\n\nBy using `:sort_maps` option, ymlr will encode all entries sorted.\n\n```elixir\niex> 1..33 |> Map.new(&{&1, &1})|> Ymlr.document!(sort_maps: true) |> IO.puts\n---\n1: 1\n2: 2\n3: 3\n...\n\n```","ref":"readme.html#encode-maps-with-keys-sorted"},{"type":"extras","title":"Benchmark - ymlr - A YAML Encoder for Elixir","doc":"This library does not claim to be particularly performant. We do have a script\nto benchmark encoding so we know if performance gets better or worse with\nchanges.\n\nYou can find the last Benchmark in [BENCHMARK.md](BENCHMARK.md)\n\n#","ref":"readme.html#benchmark"},{"type":"extras","title":"Running the script - ymlr - A YAML Encoder for Elixir","doc":"```bash\ncd benchmark/\nelixir run.exs\n```","ref":"readme.html#running-the-script"},{"type":"extras","title":"Usage","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Usage\n\n```elixir\nMix.install([:kino, :ymlr, :yaml_elixir])\n\nfull = %{\n  \"atom\" => :ymlr,\n  \"date\" => ~D[2018-11-15],\n  \"date_time\" => ~U[2018-11-15 11:00:00Z],\n  \"list\" => [\"item 1\", \"item 2\", \"item 3\"],\n  \"nested list\" => [[\"item 1.1\", \"item 1.2\"], [\"item 2.1\", \"item 2.2\", \"item 2.3\"]],\n  \"nested map\" => %{\n    \"some\" => %{\n      \"nested\" => [\"data\"]\n    }\n  },\n  \"numbers\" => %{\n    \"float\" => 3.14159265,\n    \"hex\" => 0x0B1010,\n    \"int\" => 123,\n    \"oct\" => 0o777\n  },\n  \"numbers as strings\" => %{\n    \"float\" => \"3.14159265\",\n    \"hex\" => \"0x0b1010\",\n    \"int\" => \"123\",\n    \"oct\" => \"0o777\"\n  },\n  \"string\" => \"hello world\",\n  \"string (multiline)\" => \"hello\\nworld\"\n}\n\nsimple_1 = %{\"message\" => \"hello world\"}\nsimple_2 = %{message: \"nice to be here\"}\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nResolving Hex dependencies...\nDependency resolution completed:\nNew:\n  kino 0.9.2\n  table 0.1.2\n  yamerl 0.10.0\n  yaml_elixir 2.9.0\n  ymlr 3.0.1\n* Getting kino (Hex package)\n* Getting ymlr (Hex package)\n* Getting yaml_elixir (Hex package)\n* Getting yamerl (Hex package)\n* Getting table (Hex package)\n==> table\nCompiling 5 files (.ex)\nGenerated table app\n==> ymlr\nCompiling 2 files (.ex)\nGenerated ymlr app\n==> kino\nCompiling 39 files (.ex)\nGenerated kino app\n===> Analyzing applications...\n===> Compiling yamerl\n==> yaml_elixir\nCompiling 6 files (.ex)\nGenerated yaml_elixir app\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"usage.html"},{"type":"extras","title":"Encoding a single document - Usage","doc":"[![Run in Livebook](https://livebook.dev/badge/v1/pink.svg)](https://livebook.dev/run?url=https%3A%2F%2Fgithub.com%2Fufirstgroup%2Fymlr%2Fblob%2Fmain%2Fusage.livemd)\n\nNote: We defined document data as variables `full`, `simple_1` and `simple_2` in the setup block at the top of this livebook.\n\n<!-- livebook:{\"reevaluate_automatically\":true} -->\n\n```elixir\nfull\n|> Ymlr.document!()\n|> IO.puts()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n---\natom: ymlr\ndate:\n  2018-11-15\ndate_time:\n  2018-11-15T11:00:00Z\nlist:\n  - item 1\n  - item 2\n  - item 3\nnested list:\n  - - item 1.1\n    - item 1.2\n  - - item 2.1\n    - item 2.2\n    - item 2.3\nnested map:\n  some:\n    nested:\n      - data\nnumbers:\n  float: 3.14159265\n  hex: 725008\n  int: 123\n  oct: 511\nnumbers as strings:\n  float: '3.14159265'\n  hex: '0x0b1010'\n  int: '123'\n  oct: '0o777'\nstring: hello world\nstring (multiline): |-\n  hello\n  world\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n<!-- livebook:{\"reevaluate_automatically\":true} -->\n\n```elixir\nsimple_1\n|> Ymlr.document()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok, \"---\\nmessage: hello world\\n\"}\n```","ref":"usage.html#encoding-a-single-document"},{"type":"extras","title":"Encoding multiple documents - Usage","doc":"```elixir\n[simple_1, simple_2]\n|> Ymlr.documents!()\n|> IO.puts()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n---\nmessage: hello world\n\n---\nmessage: nice to be here\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n<!-- livebook:{\"reevaluate_automatically\":true} -->\n\n```elixir\n[simple_1, simple_2]\n|> Ymlr.documents()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok, \"---\\nmessage: hello world\\n\\n---\\nmessage: nice to be here\\n\"}\n```","ref":"usage.html#encoding-multiple-documents"},{"type":"extras","title":"Adding comments - Usage","doc":"To add a comment at the top of the YAML document, pass a tuple in the form `{comment, document}` to any of the functions.\n\n<!-- livebook:{\"reevaluate_automatically\":true} -->\n\n```elixir\n{\"Single comment\", simple_1}\n|> Ymlr.document!()\n|> IO.puts()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n---\n# Single comment\nmessage: hello world\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\nThe first element of the tuple can be a list of comments:\n\n<!-- livebook:{\"reevaluate_automatically\":true} -->\n\n```elixir\n{[\"First comment\", \"Second Comment\", \"Third Comment\"], simple_1}\n|> Ymlr.document!()\n|> IO.puts()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n---\n# First comment\n# Second Comment\n# Third Comment\nmessage: hello world\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\nMultiple documents with comments:\n\n```elixir\n[\n  {[\"First doc, first comment\", \"First doc, second Comment\", \"First doc,  Comment\"], simple_1},\n  {[\"Second doc, first comment\", \"Second doc, second comment\"], simple_2}\n]\n|> Ymlr.documents!()\n|> IO.puts()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n---\n# First doc, first comment\n# First doc, second Comment\n# First doc,  Comment\nmessage: hello world\n\n---\n# Second doc, first comment\n# Second doc, second comment\nmessage: nice to be here\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"usage.html#adding-comments"},{"type":"extras","title":"Encoding Atom Map Keys - Usage","doc":"By default, atoms as map keys are encoded as strings (without the leading\ncolon). If you want atoms to be encoded with a leading colon in order to be able\nto parse it later using [`YamlElixir`'s `atoms`\noption](https://hexdocs.pm/yaml_elixir/readme.html#support-for-atoms), you can\npass `atoms: true` as second argument to any of the `Ymlr` module's functions:\n\n```elixir\nsimple_2\n|> Ymlr.document!(atoms: true)\n|> YamlElixir.read_from_string!(atoms: true)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{message: \"nice to be here\"}\n```","ref":"usage.html#encoding-atom-map-keys"},{"type":"extras","title":"Encode maps with keys sorted - Usage","doc":"Maps in elixir, implemented by erlang `:maps`, internally are `flatmap`s or `hashmap`s by size.\nLarge maps will be encoded in strange order.\n\n```elixir\nmap = Map.new(1..33, &{&1, &1})\nmap |> Ymlr.document!() |> IO.puts()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n---\n4: 4\n25: 25\n8: 8\n...\n\n```\n\nBy using `:sort_maps` option, ymlr will encode all entries sorted.\n\n```elixir\nmap |> Ymlr.document!(sort_maps: true) |> IO.puts()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n---\n1: 1\n2: 2\n3: 3\n...\n\n```","ref":"usage.html#encode-maps-with-keys-sorted"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/).","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"<!-- Add your changelog entry to the relevant subsection -->\n\n<!-- #","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added | Changed | Deprecated | Removed | Fixed | Security --> - Changelog","doc":"<!--------------------- Don't add new entries after this line --------------------->\n\n## [5.1.0] - 2023-12-27\n\n#","ref":"changelog.html#added-changed-deprecated-removed-fixed-security"},{"type":"extras","title":"Added - Changelog","doc":"- Support for escape and unicode characters [#98](https://github.com/ufirstgroup/ymlr/pull/98)\n- Refactor `Ymlr.Encode` to make it faster [#171](https://github.com/ufirstgroup/ymlr/pull/171).\n- Handle special floats `.Inf` and `.Nan`. [#170](https://github.com/ufirstgroup/ymlr/issues/170), [#172](https://github.com/ufirstgroup/ymlr/issues/172)\n- Script for benchmarking\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Put single quotes around indicator chars when it is the entire value. [#177](https://github.com/ufirstgroup/ymlr/issues/177), [#178](https://github.com/ufirstgroup/ymlr/pull/178)\n\n## [5.0.0] - 2023-11-03\n\nAlthough this release contains only fixes but as the encoding changes, we\nconsider it a breaking change. Hence the major bump.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- handle \"\\n\" strings - [#157](https://github.com/ufirstgroup/ymlr/issues/157),[#159](https://github.com/ufirstgroup/ymlr/pull/159)\n- handle empty list - [#163](https://github.com/ufirstgroup/ymlr/pull/163)\n\n## [4.2.0] - 2023-08-18\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Adds an option `sort_maps` to encode map with entries sorted - [#139](https://github.com/ufirstgroup/ymlr/issues/139),[#144](https://github.com/ufirstgroup/ymlr/pull/144)\n- Quote values beginning with `:{` - [#140](https://github.com/ufirstgroup/ymlr/issues/140),[#141](https://github.com/ufirstgroup/ymlr/pull/141)\n\n## [4.1.0] - 2023-04-23\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `Ymlr.Encoder` - Option `except: :defaults` to exclude default values\n\n## [4.0.0] - 2023-04-16\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Breaking Change - Changelog","doc":"**Up to and including version 3.x, `ymlr` would encode structs out of the box.\nThis version introduces protocols so and structs need to `@derive Ymlr.Encoder`\nin order to be encoded. See the documentation of the `Ymlr.Encoder` protocol\nfor further information.**\n\n#","ref":"changelog.html#breaking-change"},{"type":"extras","title":"Changed - Changelog","doc":"- A protocol based implementation was added which give more freedom to users of this library - [#118](https://github.com/ufirstgroup/ymlr/pull/118)\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Adds an option `atoms` to encode atom map keys with a leading colon.\n- Tuples are now encoded as lists.\n\n## [3.0.1] - 2022-09-05\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Wrap map keys in doouble quotes when required ([#94](https://github.com/ufirstgroup/ymlr/issues/94), [#95](https://github.com/ufirstgroup/ymlr/pull/95))\n- Encode structs by turning them to lists before mapping over them\n\n## [3.0.0] - 2022-08-07\n\n**In this release we changed the way `DateTime` is encoded (see below). This can be a breaking change if you rely on the old date format with spaces. Because of this change, version 3.0.0 is now again compatible with Elixir 1.10**\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- use `Enum.map_join/3` indead of `Enum.map/2` and `Enum.join/2` as it's more efficient according to credo recommendations\n- Change the serialization of timestamps to use the canonical (iso8601) format, i.e. before: `2022-07-31 14:48:48.000000000 Z` and now: `\"2022-07-31T14:48:48Z\"` ([#87](https://github.com/ufirstgroup/ymlr/issues/87), [#90](https://github.com/ufirstgroup/ymlr/pull/90))\n\n## [2.0.0] - 2021-04-02\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Removed - Changelog","doc":"- 2.0 and upwards don't support Elixir 1.10 anymore. Use version 1.x for Elixir 1.10 support.\n\n#","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Date and DateTime support (#17)\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Chores - Changelog","doc":"- yaml_elixir upgraded to 2.6.0\n- excoveralls upgraded to 0.14.0\n- ex_doc upgraded to 0.24.1\n- credo upgraded to 1.5.5\n- dialyxir upgraded to 1.1.0\n\n## [1.1.0] - 2021-04-02\n\n#","ref":"changelog.html#chores"},{"type":"extras","title":"Added - Changelog","doc":"- Date and DateTime support (#17)\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Chores - Changelog","doc":"- yaml_elixir upgraded to 2.6.0\n- excoveralls upgraded to 0.14.0\n- ex_doc upgraded to 0.24.1\n- credo upgraded to 1.5.5\n- dialyxir upgraded to 1.1.0\n\n## [1.0.1] - 2020-09-22\n\n#","ref":"changelog.html#chores"},{"type":"extras","title":"Changed - Changelog","doc":"- Rescue ArgumentError exception for oversize floats according to Float.parse/1 doc\n\n## [1.0.0] - 2020-08-21\n\nNo changes in this release. We have tested the library on a big bunch of CRDs and feel confident to publish a sable relese.\n\n## [0.0.1] - 2020-07-31\n\nFirst ymlr beta release","ref":"changelog.html#changed"},{"type":"extras","title":"BENCHMARK","doc":"Benchmark\n\nBenchmark run from 2023-11-23 21:52:00.638488Z UTC","ref":"benchmark.html"},{"type":"extras","title":"System - BENCHMARK","doc":"Benchmark suite executing on the following system:\n\n \n   \n     Operating System \n     macOS \n    \n     CPU Information \n     Apple M2 Max \n    \n     Number of Available Cores \n     12 \n    \n     Available Memory \n     64 GB \n    \n     Elixir Version \n     1.15.7 \n    \n     Erlang Version \n     26.1.2","ref":"benchmark.html#system"},{"type":"extras","title":"Configuration - BENCHMARK","doc":"Benchmark suite executing with the following configuration:\n\n \n   \n     :time \n     5 s \n    \n     :parallel \n     1 \n    \n     :warmup \n     2 s","ref":"benchmark.html#configuration"},{"type":"extras","title":"Statistics - BENCHMARK","doc":"__Input: CITM Catalog__\n\nRun Time\n\n \n   \n     Name \n     IPS \n     Average \n     Devitation \n     Median \n     99th&nbsp;% \n   \n\n   \n     Jason \n     271.78 \n     3.68 ms \n     &plusmn;14.00% \n     3.44 ms \n     4.95 ms \n   \n\n   \n     Ymlr \n     70.89 \n     14.11 ms \n     &plusmn;3.32% \n     14.06 ms \n     15.51 ms \n   \n\n \n\n\nRun Time Comparison\n\n \n   \n     Name \n     IPS \n     Slower \n   \n     Jason \n     271.78 \n     &nbsp; \n   \n\n   \n     Ymlr \n     70.89 \n     3.83x \n   \n\n \n\n\n\n\n__Input: Canada__\n\nRun Time\n\n \n   \n     Name \n     IPS \n     Average \n     Devitation \n     Median \n     99th&nbsp;% \n   \n\n   \n     Jason \n     124.25 \n     8.05 ms \n     &plusmn;14.06% \n     7.67 ms \n     13.61 ms \n   \n\n   \n     Ymlr \n     30.69 \n     32.58 ms \n     &plusmn;8.57% \n     33.61 ms \n     36.95 ms \n   \n\n \n\n\nRun Time Comparison\n\n \n   \n     Name \n     IPS \n     Slower \n   \n     Jason \n     124.25 \n     &nbsp; \n   \n\n   \n     Ymlr \n     30.69 \n     4.05x \n   \n\n \n\n\n\n\n__Input: Twitter__\n\nRun Time\n\n \n   \n     Name \n     IPS \n     Average \n     Devitation \n     Median \n     99th&nbsp;% \n   \n\n   \n     Jason \n     372.32 \n     2.69 ms \n     &plusmn;4.89% \n     2.67 ms \n     3.22 ms \n   \n\n   \n     Ymlr \n     123.21 \n     8.12 ms \n     &plusmn;1.95% \n     8.11 ms \n     8.59 ms \n   \n\n \n\n\nRun Time Comparison\n\n \n   \n     Name \n     IPS \n     Slower \n   \n     Jason \n     372.32 \n     &nbsp; \n   \n\n   \n     Ymlr \n     123.21 \n     3.02x","ref":"benchmark.html#statistics"}],"content_type":"text/markdown"}